PROJECT_NAME = 'quintagroup.seoptimizer'
SKINS_DIR = 'skins'

GLOBALS = globals()

PROPERTY_SHEET = 'seo_properties'
SHEET_TITLE = 'SEO Properties'

STOP_WORDS = ['a', 'an', 'amp', 'and', 'are', 'arial', 'as', 'at', 'be', 'but', 'by', 'can', 'com', 'do', 'font',
              'for', 'from', 'gif', 'had', 'has', 'have', 'he', 'helvetica', 'her', 'his', 'how', 'href', 'i',
              'if', 'in', 'is', 'it', 'javascript', 'jpg', 'made', 'net', 'of', 'on', 'or', 'org', 'our', 'sans',
              'see', 'serif', 'she', 'that', 'the', 'this', 'to', 'us', 'we', 'with', 'you', 'your']

FIELDS = ['seo_title', 'seo_description', 'seo_keywords']

DEFAULT_CUSTOM_METATAGS = []
